@@ -2,11 +2,12 @@
 
 from __future__ import (absolute_import, division, print_function, unicode_literals)
 
-__version__ = "0.1.2"
+__version__ = "0.1.5"
 
 from gridded.gridded import Dataset
 from gridded.grids import Grid
 from gridded.variable import Variable, VectorVariable
+from gridded.time import Time
 
 from gridded.depth import DepthBase
 DepthBase._default_component_types['variable'] = Variable
@@ -14,4 +15,5 @@ DepthBase._default_component_types['variable'] = Variable
 __all__ = ["Variable",
            "VectorVariable",
            "Grid",
-           "Dataset"]
+           "Dataset",
+           "Time"]
